
# Lab 03: Mitigation

TL;DR We have discussed different mitigations during this course. The main
mitigation that is about to see deployment is Control-Flow Integrity (CFI).
In this lab, we will experiment with C and C++ to discover the limites of CFI.

In this lab we will experiment with LLVM-based CFI sanitization. LLVM since
version 4.0 provides CFI sanitization through `-fsanitize=cfi` when combined
with `-flto` (as CFI checks are enabled during link time optimization). Read
the [LLVM-CFI docs](https://clang.llvm.org/docs/ControlFlowIntegrity.html).

Again, send the writeup to [Mathias Payer](mailto:mathias.payer@epfl.ch) with
a short summary and explanation on what you found for both tasks.

## Task 0: Skeleton

CFI protects against rogue memory writes through arbitrary memory corruption.
To experiment with different CFI settings we therefore have to start with a
corruptible program. To prepare for this lab, you therefore have to extend a
skeleton that allows arbitrary corruption of a function pointer (for Task 1)
or a vtable pointer (for Task 2).

A simple way to enable the different attacks is to leak the base address of
the module (e.g., the address of the first function) and then look at the
offsets between functions to enable attacking of non-address taken functions.
An address taken function is a function whose address was taken through the
address-of operator: `(void)(*ptr)() = &foo;`; non-address taken functions are
therefore functions whose address has *not* been taken. You can determine the
offsets between functions by looking at the compiled code (e.g., through
`objdump -d`).

Make yourself familiar with the skeleton and make sure that you understand it.
Test the sekeleton by overwriting the function pointer with `&notcalled()`.


## Task 1: CFI for C

Write a C program that uses the write-what-where vulnerability to overwrite the
function pointer. Implement a set of small test functions with different
function prototypes. Call these functions through the function pointer.
A note: LLVM-CFI considerd `void func()` and `void func(void)` to be of
different type.

* Can you switch control-flow to a function with the same prototype (the same
  set and types of arguments)?

* Can you switch to a function with a different return type?

* Can you switch to a function with a different number of arguments?

* Can you switch to a function with the same number of arguments but with
  different types?

* What about variadic functions?

* Can you switch control-flow to non-address taken functions? 

Record your findings in a table that specifies the different targets that
remain allowed. Discuss what the different options change.


## Task 2: CFI for C++

CFI also protects C++ programs by ensuring that the vtable pointer is valid
according to the expected program. Test the power of CFI by evaluating
different class hierarchies.

* Does CFI detect an overwrite along the class hierarchy?

* Does CFI detect an overwrite up the class hierarchy?

* What about the difference between declared class of the reference and the
  underlying class type?

* What about an unrelated class?

Record your findings in a table that specifies the different targets that
remain allowed. Discuss what the different options change.